//+------------------------------------------------------------------+
//|                                                     StochRSI.mq4 |
//|                                                                  |
//| Stochastic RSI                                                   |
//|                                                                  |
//| Algorithm taken from book "The New Technical Trader" by T Chande |
//|                                                                  |
//|                                 original code by Witold Wozniak" |
//|                                          http://www.mqlsoft.com/ |
//|                                                                  |
//|                                        adapted code by Linuxser  |
//|                                       http://www.linuxser.com.ar |
//|                                         http://www.forex-tsd.com |
//|                                                    November 2008 |
//+------------------------------------------------------------------+
#property copyright "Linuxser under GNU V2 license"
#property link      "www.linuxser.com.ar and www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Red

#property indicator_level1 80
#property indicator_level2 50
#property indicator_level3 20
#property indicator_minimum -2
#property indicator_maximum 102

double StocRSI[];
double Value3[];

//---- input parameters
/*************************************************************************
PRICE_CLOSE    0 Close price. 
PRICE_OPEN     1 Open price. 
PRICE_HIGH     2 High price. 
PRICE_LOW      3 Low price. 
PRICE_MEDIAN   4 Median price, (high+low)/2. 
PRICE_TYPICAL  5 Typical price, (high+low+close)/3. 
PRICE_WEIGHTED 6 Weighted close price, (high+low+close+close)/4. 
You must use the numeric value of the Applied Price that you want to use
when you set the 'applied_price' value with the indicator inputs.
**************************************************************************/

extern int RSILength = 14;
extern int PriceMode = 0;
int buffers = 0;
int drawBegin = 0;
string fast_name;

int init() {
    drawBegin = RSILength;
    initBuffer(StocRSI, "Stochastic RSI", DRAW_LINE);    
    initBuffer(Value3);
    IndicatorBuffers(buffers);
    
   switch(PriceMode)
   {
   case 1 : fast_name="Open"; break;
   case 2 : fast_name="High"; break;
   case 3 : fast_name="Low"; break;
   case 4 : fast_name="Median"; break;
   case 5 : fast_name="Typical"; break;
   case 6 : fast_name="Weighted"; break;
   default: PriceMode=0; fast_name="Close";
   }
    
    IndicatorShortName("Stochastic RSI [" + RSILength + ", "+fast_name+"]");
    return (0);
}

int start() {
    if (Bars <= drawBegin) return (0);
    int countedBars = IndicatorCounted();
    if (countedBars < 0) return (-1);
    if (countedBars > 0) countedBars--;
    int s, limit = Bars - countedBars - 1;
    for (s = limit; s >= 0; s--) {
        double rsi = iRSI(0, 0, RSILength, PriceMode, s);
        double hh = rsi, ll = rsi;
        for (int i = 0; i < RSILength; i++) {
            double tmp = iRSI(0, 0, RSILength, PriceMode, s + i);
            hh = MathMax(hh, tmp);
            ll = MathMin(ll, tmp);
        }
        double Value1 = rsi - ll;
        double Value2 = hh - ll;
        Value3[s] = 0.0;
        if (Value2 != 0.0) {
            Value3[s] = 100* (Value1 / Value2);
        }
        
    }
    for (s = limit - 1; s >= 0; s--) {
        StocRSI[s] = Value3[s];        
    }
    return (0);   
}

void initBuffer(double array[], string label = "", int type = DRAW_NONE, int arrow = 0, int style = EMPTY, int width = EMPTY, color clr = CLR_NONE) {
    SetIndexBuffer(buffers, array);
    SetIndexLabel(buffers, label);
    SetIndexEmptyValue(buffers, EMPTY_VALUE);
    SetIndexDrawBegin(buffers, drawBegin);
    SetIndexShift(buffers, 0);
    SetIndexStyle(buffers, type, style, width);
    SetIndexArrow(buffers, arrow);
    buffers++;
}